// *********************************************************************
// *                          SPI master                               *
// *                         Demo example                              *
// *********************************************************************
//
// Intended for:
//   HW: - TR-72G + DK-EVAL-04x development kit
//
//   OS: - v4.06G
//
// Description:
//   - example shows how to use SPI peripheral of MCU as an SPI master
//   - connect logic analyzer to the DK-EVAL-04x to check the SPI communication:
//      TR module pin   DK-EVAL-04x pin     SPI
//      --------------------------------------- 
//      C8              1                   SDO
//      C7              2                   SDI
//      C6              3                   SCK
//      C5              4                   SS
//      C4              7                   GND
//
// ---------------------------------------------------------------------
// Online IQRF OS Reference Guide: https://doc.iqrf.org/IQRF-OS-Reference-guide
//
// Copyright (c) IQRF Tech s.r.o.
//
// File:    SPI_MASTER.c
// Version: v3.00                                   Revision: 18/06/2021
//
// Revision history:
//   v3.00: 18/06/2021  First release for TR-7xG.
//
// *********************************************************************
// Default IQRF include (modify the path according to your setup)
#include "IQRF.h"

// *********************************************************************
#define SS_PIN      PORTA.5

// *********************************************************************
void masterSPIinit(void);
uns8 sendSPIbyte(uns8 tx_byte);
void sendSPIdata(uns8 data_len);

// *********************************************************************
void APPLICATION()
{
    setWDTon_4s();
    pulseLEDG();
    masterSPIinit();
    
    while (1)
    {
        clrwdt();
        appInfo();              // Copy Application data from EEPROM to bufferINFO
        copyBufferINFO2COM();   //   and then copy it to bufferCOM
        sendSPIdata(15);        // Send/receive 15 B via SPI from/to bufferCOM  
        waitDelay(10);
    }
}
// *********************************************************************

void masterSPIinit(void)
{
    // Initialize PINs to inputs and outputs
    TRISC.5 = 0;                // RC5 as output SDO (C8)
    TRISC.4 = 1;                // RC4 as input SDI (C7)
    TRISC.3 = 0;                // RC3 as output SCK (C6)
    TRISA.5 = 0;                // RA5 as output SS (C5)
    
    moduleInfo();

    if (bufferINFO[5].7 == 0)   // TR module with connected pins?
    {                           // Yes
        TRISC.6 = 1;            // RC6 as input (connected to RA5 in parallel)
        TRISB.4 = 1;            // RB4 as input (connected to RA5 in parallel)
        TRISC.7 = 1;            // RC7 as input (connected to RC5 in parallel)
    }
                                  
    SS_PIN = 1;                 // Set idle level of SS
    
    // PPS setting
    unlockPPS();
    SSP1DATPPS = 0x14;          // In:  SDI1 - RC4
    RC3PPS = 0x14;              // Out: SCK1 - RC3
    RC5PPS = 0x15;              // Out: SDO1 - RC5
    lockPPS();
    
    // SPI setting
    MSSP1MD = 0;                // Enable MSSP1 module
    // SSP1STAT:
    //  Transmit occurs on SCK rising edge, CKE
    SSP1STAT.6 = 1;
    // SSP1CON1:
    //  SPI enabled
    //  Idle state for clock is a low level
    //  CLK = 250kHz @ 16 MHz CPU clock
    SSP1CON1 = 0b0.0.1.0.0010;
}
// *********************************************************************

// Send/receive data_len bytes via SPI from/to bufferCOM
void sendSPIdata(uns8 data_len) 
{
uns8 i, val;

    SS_PIN = 0;                 // SS - active
    
    for (i = 0; i < data_len; i++)
    {
        val = sendSPIbyte(readFromRAM(bufferCOM + i));
        writeToRAM(bufferCOM + i, val);
    }
    
    SS_PIN = 1;                 // SS - deactive
}
// *********************************************************************

// Send/receive 1 B via SPI
uns8 sendSPIbyte(uns8 tx_byte)
{
    SSP1BUF = tx_byte;
    while (!SSP1IF);                // Wait until the byte is transmited/received
    SSP1IF = 0;
    return SSP1BUF;             
}
// *********************************************************************

#pragma packedCdataStrings 0    // Store the string unpacked
                                //   (one byte in one location)
#pragma cdata[__EEAPPINFO] =  "SPI master demo"
                                // Stored to the Application Info array in EEPROM
// *********************************************************************

